@@ -46,10 +46,9 @@ from sqlglot.schema import MappingSchema as MappingSchema, Schema as Schema
 from sqlglot.tokens import Tokenizer as Tokenizer, TokenType as TokenType
 
 if t.TYPE_CHECKING:
+    from sqlglot._typing import E
     from sqlglot.dialects.dialect import DialectType as DialectType
 
-    T = t.TypeVar("T", bound=Expression)
-
 logger = logging.getLogger("sqlglot")
 
 
@@ -85,56 +84,19 @@ def parse(sql: str, read: DialectType = None, **opts) -> t.List[t.Optional[Expre
 
 
 @t.overload
-def parse_one(
-    sql: str,
-    read: None = None,
-    into: t.Type[T] = ...,
-    **opts,
-) -> T:
+def parse_one(sql: str, *, into: t.Type[E], **opts) -> E:
     ...
 
 
 @t.overload
-def parse_one(
-    sql: str,
-    read: DialectType,
-    into: t.Type[T],
-    **opts,
-) -> T:
-    ...
-
-
-@t.overload
-def parse_one(
-    sql: str,
-    read: None = None,
-    into: t.Union[str, t.Collection[t.Union[str, t.Type[Expression]]]] = ...,
-    **opts,
-) -> Expression:
-    ...
-
-
-@t.overload
-def parse_one(
-    sql: str,
-    read: DialectType,
-    into: t.Union[str, t.Collection[t.Union[str, t.Type[Expression]]]],
-    **opts,
-) -> Expression:
-    ...
-
-
-@t.overload
-def parse_one(
-    sql: str,
-    **opts,
-) -> Expression:
+def parse_one(sql: str, **opts) -> Expression:
     ...
 
 
 def parse_one(
     sql: str,
     read: DialectType = None,
+    dialect: DialectType = None,
     into: t.Optional[exp.IntoType] = None,
     **opts,
 ) -> Expression:
@@ -144,6 +106,7 @@ def parse_one(
     Args:
         sql: the SQL code string to parse.
         read: the SQL dialect to apply during parsing (eg. "spark", "hive", "presto", "mysql").
+        dialect: the SQL dialect (alias for read)
         into: the SQLGlot Expression to parse into.
         **opts: other `sqlglot.parser.Parser` options.
 
@@ -151,7 +114,7 @@ def parse_one(
         The syntax tree for the first parsed statement.
     """
 
-    dialect = Dialect.get_or_raise(read)()
+    dialect = Dialect.get_or_raise(read or dialect)()
 
     if into:
         result = dialect.parse_into(into, sql, **opts)
