@@ -1,9 +1,18 @@
+from __future__ import annotations
+
 import argparse
+import sys
+import typing as t
 
 import sqlglot
 
 parser = argparse.ArgumentParser(description="Transpile SQL")
-parser.add_argument("sql", metavar="sql", type=str, help="SQL string to transpile")
+parser.add_argument(
+    "sql",
+    metavar="sql",
+    type=str,
+    help="SQL statement(s) to transpile, or - to parse stdin.",
+)
 parser.add_argument(
     "--read",
     dest="read",
@@ -36,6 +45,12 @@ parser.add_argument(
     action="store_true",
     help="Parse and return the expression tree",
 )
+parser.add_argument(
+    "--tokenize",
+    dest="tokenize",
+    action="store_true",
+    help="Tokenize and return the tokens list",
+)
 parser.add_argument(
     "--error-level",
     dest="error_level",
@@ -48,11 +63,22 @@ parser.add_argument(
 args = parser.parse_args()
 error_level = sqlglot.ErrorLevel[args.error_level.upper()]
 
+sql = sys.stdin.read() if args.sql == "-" else args.sql
+
 if args.parse:
-    sqls = [repr(expression) for expression in sqlglot.parse(args.sql, read=args.read, error_level=error_level)]
+    objs: t.Union[t.List[str], t.List[sqlglot.tokens.Token]] = [
+        repr(expression)
+        for expression in sqlglot.parse(
+            sql,
+            read=args.read,
+            error_level=error_level,
+        )
+    ]
+elif args.tokenize:
+    objs = sqlglot.Dialect.get_or_raise(args.read)().tokenize(sql)
 else:
-    sqls = sqlglot.transpile(
-        args.sql,
+    objs = sqlglot.transpile(
+        sql,
         read=args.read,
         write=args.write,
         identify=args.identify,
@@ -60,5 +86,5 @@ else:
         error_level=error_level,
     )
 
-for sql in sqls:
-    print(sql)
+for obj in objs:
+    print(obj)
