@@ -747,11 +747,11 @@ def ascii(col: ColumnOrLiteral) -> Column:
 
 
 def base64(col: ColumnOrLiteral) -> Column:
-    return Column.invoke_anonymous_function(col, "BASE64")
+    return Column.invoke_expression_over_column(col, expression.ToBase64)
 
 
 def unbase64(col: ColumnOrLiteral) -> Column:
-    return Column.invoke_anonymous_function(col, "UNBASE64")
+    return Column.invoke_expression_over_column(col, expression.FromBase64)
 
 
 def ltrim(col: ColumnOrName) -> Column:
@@ -1119,7 +1119,7 @@ def map_entries(col: ColumnOrName) -> Column:
 
 
 def map_from_entries(col: ColumnOrName) -> Column:
-    return Column.invoke_anonymous_function(col, "MAP_FROM_ENTRIES")
+    return Column.invoke_expression_over_column(col, expression.MapFromEntries)
 
 
 def array_repeat(col: ColumnOrName, count: t.Union[ColumnOrName, int]) -> Column:
