@@ -124,7 +124,9 @@ def remove_precision_parameterized_types(expression: exp.Expression) -> exp.Expr
     other expressions. This transforms removes the precision from parameterized types in expressions.
     """
     for node in expression.find_all(exp.DataType):
-        node.set("expressions", [e for e in node.expressions if isinstance(e, exp.DataType)])
+        node.set(
+            "expressions", [e for e in node.expressions if not isinstance(e, exp.DataTypeSize)]
+        )
 
     return expression
 
@@ -157,14 +159,10 @@ def unnest_to_explode(expression: exp.Expression) -> exp.Expression:
 def explode_to_unnest(expression: exp.Expression) -> exp.Expression:
     """Convert explode/posexplode into unnest (used in hive -> presto)."""
     if isinstance(expression, exp.Select):
-        from sqlglot.optimizer.scope import build_scope
-
-        scope = build_scope(expression)
-        if not scope:
-            return expression
+        from sqlglot.optimizer.scope import Scope
 
         taken_select_names = set(expression.named_selects)
-        taken_source_names = set(scope.selected_sources)
+        taken_source_names = {name for name, _ in Scope(expression).references}
 
         for select in expression.selects:
             to_replace = select
@@ -219,25 +217,6 @@ def explode_to_unnest(expression: exp.Expression) -> exp.Expression:
     return expression
 
 
-def remove_target_from_merge(expression: exp.Expression) -> exp.Expression:
-    """Remove table refs from columns in when statements."""
-    if isinstance(expression, exp.Merge):
-        alias = expression.this.args.get("alias")
-        targets = {expression.this.this}
-        if alias:
-            targets.add(alias.this)
-
-        for when in expression.expressions:
-            when.transform(
-                lambda node: exp.column(node.name)
-                if isinstance(node, exp.Column) and node.args.get("table") in targets
-                else node,
-                copy=False,
-            )
-
-    return expression
-
-
 def remove_within_group_for_percentiles(expression: exp.Expression) -> exp.Expression:
     if (
         isinstance(expression, exp.WithinGroup)
