@@ -218,7 +218,7 @@ class TestDialect(Validator):
                 "presto": "CAST(a AS SMALLINT)",
                 "redshift": "CAST(a AS SMALLINT)",
                 "snowflake": "CAST(a AS SMALLINT)",
-                "spark": "CAST(a AS SHORT)",
+                "spark": "CAST(a AS SMALLINT)",
                 "sqlite": "CAST(a AS INTEGER)",
                 "starrocks": "CAST(a AS SMALLINT)",
             },
@@ -601,7 +601,6 @@ class TestDialect(Validator):
             "DATE_TRUNC('day', x)",
             read={
                 "bigquery": "DATE_TRUNC(x, day)",
-                "duckdb": "DATE_TRUNC('day', x)",
                 "spark": "TRUNC(x, 'day')",
             },
             write={
@@ -619,6 +618,7 @@ class TestDialect(Validator):
             "TIMESTAMP_TRUNC(x, day)",
             read={
                 "bigquery": "TIMESTAMP_TRUNC(x, day)",
+                "duckdb": "DATE_TRUNC('day', x)",
                 "presto": "DATE_TRUNC('day', x)",
                 "postgres": "DATE_TRUNC('day', x)",
                 "snowflake": "DATE_TRUNC('day', x)",
@@ -1307,7 +1307,7 @@ class TestDialect(Validator):
             write={
                 "sqlite": "SELECT x FROM y LIMIT 10",
                 "oracle": "SELECT x FROM y FETCH FIRST 10 ROWS ONLY",
-                "tsql": "SELECT x FROM y FETCH FIRST 10 ROWS ONLY",
+                "tsql": "SELECT TOP 10 x FROM y",
             },
         )
         self.validate_all(
@@ -1413,7 +1413,8 @@ class TestDialect(Validator):
                 "presto": "SELECT a AS b FROM x GROUP BY 1",
                 "hive": "SELECT a AS b FROM x GROUP BY 1",
                 "oracle": "SELECT a AS b FROM x GROUP BY 1",
-                "spark": "SELECT a AS b FROM x GROUP BY 1",
+                "spark": "SELECT a AS b FROM x GROUP BY b",
+                "spark2": "SELECT a AS b FROM x GROUP BY 1",
             },
         )
         self.validate_all(
