@@ -155,6 +155,7 @@ REGEXP_LIKE('new york', '.')
 REGEXP_SPLIT('new york', '.')
 SPLIT('new york', '.')
 X((y AS z)).1
+X(a.b = 1)
 (x AS y, y AS z)
 REPLACE(1)
 DATE(x) = DATE(y)
@@ -413,6 +414,8 @@ SELECT 1 FROM a NATURAL LEFT JOIN b
 SELECT 1 FROM a NATURAL LEFT OUTER JOIN b
 SELECT 1 FROM a OUTER JOIN b ON a.foo = b.bar
 SELECT 1 FROM a FULL JOIN b ON a.foo = b.bar
+SELECT 1 FROM a JOIN b JOIN c ON b.id = c.id ON a.id = b.id
+SELECT * FROM a JOIN b JOIN c USING (id) USING (id)
 SELECT 1 UNION ALL SELECT 2
 SELECT 1 EXCEPT SELECT 2
 SELECT 1 EXCEPT SELECT 2
@@ -420,7 +423,6 @@ SELECT 1 INTERSECT SELECT 2
 SELECT 1 INTERSECT SELECT 2
 SELECT 1 AS delete, 2 AS alter
 SELECT * FROM (x)
-SELECT * FROM ((x))
 SELECT * FROM ((SELECT 1))
 SELECT * FROM (x CROSS JOIN foo LATERAL VIEW EXPLODE(y))
 SELECT * FROM (SELECT 1) AS x
@@ -519,8 +521,6 @@ SELECT student, score FROM tests CROSS JOIN UNNEST(scores) AS t(a, b)
 SELECT student, score FROM tests CROSS JOIN UNNEST(scores) WITH ORDINALITY AS t(a, b)
 SELECT student, score FROM tests CROSS JOIN UNNEST(x.scores) AS t(score)
 SELECT student, score FROM tests CROSS JOIN UNNEST(ARRAY(x.scores)) AS t(score)
-SELECT * FROM t WITH (TABLOCK, INDEX(myindex))
-SELECT * FROM t WITH (NOWAIT)
 CREATE TABLE foo AS (SELECT 1) UNION ALL (SELECT 2)
 CREATE TABLE foo (id INT PRIMARY KEY ASC)
 CREATE TABLE a.b AS SELECT 1
@@ -553,17 +553,17 @@ CREATE TABLE z AS ((WITH cte AS (SELECT 1) SELECT * FROM cte))
 CREATE TABLE z (a INT UNIQUE)
 CREATE TABLE z (a INT AUTO_INCREMENT)
 CREATE TABLE z (a INT UNIQUE AUTO_INCREMENT)
-CREATE TABLE z (a INT REFERENCES parent(b, c))
-CREATE TABLE z (a INT PRIMARY KEY, b INT REFERENCES foo(id))
-CREATE TABLE z (a INT, FOREIGN KEY (a) REFERENCES parent(b, c))
-CREATE TABLE foo (bar INT REFERENCES baz(baz_id) ON DELETE NO ACTION)
-CREATE TABLE foo (bar INT REFERENCES baz(baz_id) ON DELETE CASCADE)
-CREATE TABLE foo (bar INT REFERENCES baz(baz_id) ON DELETE SET NULL)
-CREATE TABLE foo (bar INT REFERENCES baz(baz_id) ON DELETE SET DEFAULT)
-CREATE TABLE foo (bar INT REFERENCES baz(baz_id) ON UPDATE NO ACTION)
-CREATE TABLE foo (bar INT REFERENCES baz(baz_id) ON UPDATE CASCADE)
-CREATE TABLE foo (bar INT REFERENCES baz(baz_id) ON UPDATE SET NULL)
-CREATE TABLE foo (bar INT REFERENCES baz(baz_id) ON UPDATE SET DEFAULT)
+CREATE TABLE z (a INT REFERENCES parent (b, c))
+CREATE TABLE z (a INT PRIMARY KEY, b INT REFERENCES foo (id))
+CREATE TABLE z (a INT, FOREIGN KEY (a) REFERENCES parent (b, c))
+CREATE TABLE foo (bar INT REFERENCES baz (baz_id) ON DELETE NO ACTION)
+CREATE TABLE foo (bar INT REFERENCES baz (baz_id) ON DELETE CASCADE)
+CREATE TABLE foo (bar INT REFERENCES baz (baz_id) ON DELETE SET NULL)
+CREATE TABLE foo (bar INT REFERENCES baz (baz_id) ON DELETE SET DEFAULT)
+CREATE TABLE foo (bar INT REFERENCES baz (baz_id) ON UPDATE NO ACTION)
+CREATE TABLE foo (bar INT REFERENCES baz (baz_id) ON UPDATE CASCADE)
+CREATE TABLE foo (bar INT REFERENCES baz (baz_id) ON UPDATE SET NULL)
+CREATE TABLE foo (bar INT REFERENCES baz (baz_id) ON UPDATE SET DEFAULT)
 CREATE TABLE asd AS SELECT asd FROM asd WITH NO DATA
 CREATE TABLE asd AS SELECT asd FROM asd WITH DATA
 CREATE TABLE products (x INT GENERATED BY DEFAULT AS IDENTITY)
@@ -574,7 +574,7 @@ CREATE TABLE IF NOT EXISTS customer (pk BIGINT NOT NULL GENERATED ALWAYS AS IDEN
 CREATE TABLE customer (pk BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 10 INCREMENT BY 1 MINVALUE -1 MAXVALUE 1 NO CYCLE))
 CREATE TABLE customer (pk BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY (START WITH 10))
 CREATE TABLE customer (pk BIGINT NOT NULL GENERATED ALWAYS AS IDENTITY (CYCLE))
-CREATE TABLE foo (baz_id INT REFERENCES baz(id) DEFERRABLE)
+CREATE TABLE foo (baz_id INT REFERENCES baz (id) DEFERRABLE)
 CREATE TABLE foo (baz CHAR(4) CHARACTER SET LATIN UPPERCASE NOT CASESPECIFIC COMPRESS 'a')
 CREATE TABLE foo (baz DATE FORMAT 'YYYY/MM/DD' TITLE 'title' INLINE LENGTH 1 COMPRESS ('a', 'b'))
 CREATE TABLE t (title TEXT)
@@ -649,7 +649,7 @@ ANALYZE a.y
 DELETE FROM x WHERE y > 1
 DELETE FROM y
 DELETE FROM event USING sales WHERE event.eventid = sales.eventid
-DELETE FROM event USING sales, USING bla WHERE event.eventid = sales.eventid
+DELETE FROM event USING sales, bla WHERE event.eventid = sales.eventid
 DELETE FROM event USING sales AS s WHERE event.eventid = s.eventid
 DELETE FROM event AS event USING sales AS s WHERE event.eventid = s.eventid
 PREPARE statement
@@ -795,6 +795,7 @@ ALTER TABLE a ADD FOREIGN KEY (x, y) REFERENCES bla
 SELECT partition FROM a
 SELECT end FROM a
 SELECT id FROM b.a AS a QUALIFY ROW_NUMBER() OVER (PARTITION BY br ORDER BY sadf DESC) = 1
+SELECT * FROM x WHERE a GROUP BY a HAVING b SORT BY s ORDER BY c LIMIT d
 SELECT LEFT.FOO FROM BLA AS LEFT
 SELECT RIGHT.FOO FROM BLA AS RIGHT
 SELECT LEFT FROM LEFT LEFT JOIN RIGHT RIGHT JOIN LEFT
@@ -835,3 +836,5 @@ SELECT * FROM case
 SELECT * FROM schema.case
 SELECT * FROM current_date
 SELECT * FROM schema.current_date
+SELECT /*+ SOME_HINT(foo) */ 1
+SELECT * FROM (tbl1 CROSS JOIN (SELECT * FROM tbl2) AS t1)
